Files
TheRock/rocm-libraries/projects/rocthrust/examples/transform_iterator.cu
T
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

153 lines
4.7 KiB
Plaintext

// Copyright (c) 2020-2025 Advanced Micro Devices, Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
#include <thrust/device_vector.h>
#include <thrust/functional.h>
#include <thrust/iterator/counting_iterator.h>
#include <thrust/iterator/transform_iterator.h>
#include <thrust/reduce.h>
#include <iostream>
#include <iterator>
#include <string>
#include "include/host_device.h"
// this functor clamps a value to the range [lo, hi]
template <typename T>
struct clamp
{
T lo, hi;
__host__ __device__ clamp(T _lo, T _hi)
: lo(_lo)
, hi(_hi)
{}
__host__ __device__ T operator()(T x)
{
if (x < lo)
{
return lo;
}
else if (x < hi)
{
return x;
}
else
{
return hi;
}
}
};
template <typename T>
struct simple_negate
{
__host__ __device__ T operator()(T x)
{
return -x;
}
};
template <typename Iterator>
void print_range(const std::string& name, Iterator first, Iterator last)
{
using T = typename std::iterator_traits<Iterator>::value_type;
std::cout << name << ": ";
thrust::copy(first, last, std::ostream_iterator<T>(std::cout, " "));
std::cout << "\n";
}
int main(void)
{
// clamp values to the range [1, 5]
int lo = 1;
int hi = 5;
// define some types
using Vector = thrust::device_vector<int>;
using VectorIterator = Vector::iterator;
// initialize values
Vector values(8);
values[0] = 2;
values[1] = 5;
values[2] = 7;
values[3] = 1;
values[4] = 6;
values[5] = 0;
values[6] = 3;
values[7] = 8;
print_range("values ", values.begin(), values.end());
// define some more types
using ClampedVectorIterator = thrust::transform_iterator<clamp<int>, VectorIterator>;
// create a transform_iterator that applies clamp() to the values array
ClampedVectorIterator cv_begin = thrust::make_transform_iterator(values.begin(), clamp<int>(lo, hi));
ClampedVectorIterator cv_end = cv_begin + values.size();
// now [clamped_begin, clamped_end) defines a sequence of clamped values
print_range("clamped values ", cv_begin, cv_end);
////
// compute the sum of the clamped sequence with reduce()
std::cout << "sum of clamped values : " << thrust::reduce(cv_begin, cv_end) << "\n";
////
// combine transform_iterator with other fancy iterators like counting_iterator
using CountingIterator = thrust::counting_iterator<int>;
using ClampedCountingIterator = thrust::transform_iterator<clamp<int>, CountingIterator>;
CountingIterator count_begin(0);
CountingIterator count_end(10);
print_range("sequence ", count_begin, count_end);
ClampedCountingIterator cs_begin = thrust::make_transform_iterator(count_begin, clamp<int>(lo, hi));
ClampedCountingIterator cs_end = thrust::make_transform_iterator(count_end, clamp<int>(lo, hi));
print_range("clamped sequence ", cs_begin, cs_end);
////
// combine transform_iterator with another transform_iterator
using NegatedClampedCountingIterator = thrust::transform_iterator<thrust::negate<int>, ClampedCountingIterator>;
NegatedClampedCountingIterator ncs_begin = thrust::make_transform_iterator(cs_begin, thrust::negate<int>());
NegatedClampedCountingIterator ncs_end = thrust::make_transform_iterator(cs_end, thrust::negate<int>());
print_range("negated sequence ", ncs_begin, ncs_end);
////
// when a functor does not define result_type, a third template argument must be provided
using NegatedVectorIterator = thrust::transform_iterator<simple_negate<int>, VectorIterator, int>;
NegatedVectorIterator nv_begin(values.begin(), simple_negate<int>());
NegatedVectorIterator nv_end(values.end(), simple_negate<int>());
print_range("negated values ", nv_begin, nv_end);
return 0;
}